@@ -236,7 +236,10 @@ impl From<String> for Action {
             "decreasefontsize" => Some(Action::DecreaseFontSize),
             "createwindow" => Some(Action::WindowCreateNew),
             "createtab" => Some(Action::TabCreateNew),
+            "movecurrenttabtoprev" => Some(Action::MoveCurrentTabToPrev),
+            "movecurrenttabtonext" => Some(Action::MoveCurrentTabToNext),
             "closetab" => Some(Action::TabCloseCurrent),
+            "closesplitortab" => Some(Action::CloseCurrentSplitOrTab),
             "closeunfocusedtabs" => Some(Action::TabCloseUnfocused),
             "openconfigeditor" => Some(Action::ConfigEditor),
             "selectprevtab" => Some(Action::SelectPrevTab),
@@ -247,7 +250,14 @@ impl From<String> for Action {
             "scrollhalfpagedown" => Some(Action::ScrollHalfPageDown),
             "scrolltotop" => Some(Action::ScrollToTop),
             "scrolltobottom" => Some(Action::ScrollToBottom),
+            "splitright" => Some(Action::SplitRight),
+            "splitdown" => Some(Action::SplitDown),
+            "selectnextsplit" => Some(Action::SelectNextSplit),
+            "selectprevsplit" => Some(Action::SelectPrevSplit),
+            "selectnextsplitortab" => Some(Action::SelectNextSplitOrTab),
+            "selectprevsplitortab" => Some(Action::SelectPrevSplitOrTab),
             "togglevimode" => Some(Action::ToggleViMode),
+            "togglefullscreen" => Some(Action::ToggleFullscreen),
             "none" => Some(Action::None),
             _ => None,
         };
@@ -313,7 +323,7 @@ pub enum Action {
     /// Scroll
     Scroll(i32),
 
-    /// Regex keyboard hints.
+    // Regex keyboard hints.
     // Hint(Hint),
 
     // Move vi mode cursor.
@@ -401,6 +411,12 @@ pub enum Action {
     /// Create a new Rio tab.
     TabCreateNew,
 
+    /// Move current tab to previous slot.
+    MoveCurrentTabToPrev,
+
+    /// Move current tab to next slot.
+    MoveCurrentTabToNext,
+
     /// Switch to next tab.
     SelectNextTab,
 
@@ -410,6 +426,8 @@ pub enum Action {
     /// Close tab.
     TabCloseCurrent,
 
+    CloseCurrentSplitOrTab,
+
     /// Close all other tabs (leave only the current tab).
     TabCloseUnfocused,
 
@@ -443,6 +461,24 @@ pub enum Action {
     /// Start a backward buffer search.
     SearchBackward,
 
+    /// Split horizontally
+    SplitRight,
+
+    /// Split vertically
+    SplitDown,
+
+    /// Select next split
+    SelectNextSplit,
+
+    /// Select previous split
+    SelectPrevSplit,
+
+    /// Select next split if available if not next tab
+    SelectNextSplitOrTab,
+
+    /// Select previous split if available if not previous tab
+    SelectPrevSplitOrTab,
+
     /// Allow receiving char input.
     ReceiveChar,
 
@@ -555,6 +591,7 @@ pub fn default_mouse_bindings() -> Vec<MouseBinding> {
 pub fn default_key_bindings(
     unprocessed_config_key_bindings: Vec<ConfigKeyBinding>,
     use_navigation_key_bindings: bool,
+    use_splits: bool,
     config_keyboard: ConfigKeyboard,
 ) -> Vec<KeyBinding> {
     let mut bindings = bindings!(
@@ -564,7 +601,6 @@ pub fn default_key_bindings(
         Key::Named(Paste), ~BindingMode::VI; Action::Paste;
         Key::Character("l".into()), ModifiersState::CONTROL; Action::ClearLogNotice;
         "l",  ModifiersState::CONTROL, ~BindingMode::VI; Action::Esc("\x0c".into());
-        Key::Named(Tab),  ModifiersState::SHIFT, ~BindingMode::VI; Action::Esc("\x1b[Z".into());
         Key::Named(Home),     ModifiersState::SHIFT, ~BindingMode::ALT_SCREEN; Action::ScrollToTop;
         Key::Named(End),      ModifiersState::SHIFT, ~BindingMode::ALT_SCREEN; Action::ScrollToBottom;
         Key::Named(PageUp),   ModifiersState::SHIFT, ~BindingMode::ALT_SCREEN; Action::ScrollPageUp;
@@ -584,29 +620,36 @@ pub fn default_key_bindings(
 
         // VI Mode
         Key::Named(Space), ModifiersState::ALT | ModifiersState::SHIFT; Action::ToggleViMode;
-        "i", +BindingMode::VI; Action::ToggleViMode;
+        "/", +BindingMode::VI, ~BindingMode::SEARCH; Action::SearchForward;
+        "n", +BindingMode::VI, ~BindingMode::SEARCH; SearchAction::SearchFocusNext;
+        "n",  ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH; SearchAction::SearchFocusPrevious;
+        Key::Named(Enter), +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusNext;
+        Key::Named(Enter), +BindingMode::SEARCH, +BindingMode::VI; SearchAction::SearchConfirm;
+        Key::Named(Escape), +BindingMode::SEARCH; SearchAction::SearchCancel;
+        Key::Named(Enter), ModifiersState::SHIFT, +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusPrevious;
+        "i", +BindingMode::VI, ~BindingMode::SEARCH; Action::ToggleViMode;
         "c", ModifiersState::CONTROL, +BindingMode::VI; Action::ToggleViMode;
         Key::Named(Escape), +BindingMode::VI; Action::ClearSelection;
-        "i", +BindingMode::VI; Action::ScrollToBottom;
-        "g", +BindingMode::VI; Action::ScrollToTop;
-        "g", ModifiersState::SHIFT, +BindingMode::VI; Action::ScrollToBottom;
-        "b", ModifiersState::CONTROL, +BindingMode::VI; Action::ScrollPageUp;
-        "f", ModifiersState::CONTROL, +BindingMode::VI; Action::ScrollPageDown;
-        "u", ModifiersState::CONTROL, +BindingMode::VI; Action::ScrollHalfPageUp;
-        "d", ModifiersState::CONTROL, +BindingMode::VI; Action::ScrollHalfPageDown;
-        "y", ModifiersState::CONTROL,  +BindingMode::VI; Action::Scroll(1);
-        "e", ModifiersState::CONTROL,  +BindingMode::VI; Action::Scroll(-1);
-        "y", +BindingMode::VI; Action::Copy;
-        "y", +BindingMode::VI; Action::ClearSelection;
-        "v", +BindingMode::VI; ViAction::ToggleNormalSelection;
-        "v", ModifiersState::SHIFT, +BindingMode::VI; ViAction::ToggleLineSelection;
-        "v", ModifiersState::CONTROL, +BindingMode::VI; ViAction::ToggleBlockSelection;
-        "v", ModifiersState::ALT, +BindingMode::VI; ViAction::ToggleSemanticSelection;
-        "z", +BindingMode::VI; ViAction::CenterAroundViCursor;
-        "k", +BindingMode::VI; ViMotion::Up;
-        "j", +BindingMode::VI; ViMotion::Down;
-        "h", +BindingMode::VI; ViMotion::Left;
-        "l", +BindingMode::VI; ViMotion::Right;
+        "i", +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollToBottom;
+        "g", +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollToTop;
+        "g", ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollToBottom;
+        "b", ModifiersState::CONTROL, +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollPageUp;
+        "f", ModifiersState::CONTROL, +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollPageDown;
+        "u", ModifiersState::CONTROL, +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollHalfPageUp;
+        "d", ModifiersState::CONTROL, +BindingMode::VI, ~BindingMode::SEARCH; Action::ScrollHalfPageDown;
+        "y", ModifiersState::CONTROL,  +BindingMode::VI, ~BindingMode::SEARCH; Action::Scroll(1);
+        "e", ModifiersState::CONTROL,  +BindingMode::VI, ~BindingMode::SEARCH; Action::Scroll(-1);
+        "y", +BindingMode::VI, ~BindingMode::SEARCH; Action::Copy;
+        "y", +BindingMode::VI, ~BindingMode::SEARCH; Action::ClearSelection;
+        "v", +BindingMode::VI, ~BindingMode::SEARCH; ViAction::ToggleNormalSelection;
+        "v", ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH; ViAction::ToggleLineSelection;
+        "v", ModifiersState::CONTROL, +BindingMode::VI, ~BindingMode::SEARCH; ViAction::ToggleBlockSelection;
+        "v", ModifiersState::ALT, +BindingMode::VI, ~BindingMode::SEARCH; ViAction::ToggleSemanticSelection;
+        "z", +BindingMode::VI, ~BindingMode::SEARCH; ViAction::CenterAroundViCursor;
+        "k", +BindingMode::VI, ~BindingMode::SEARCH; ViMotion::Up;
+        "j", +BindingMode::VI, ~BindingMode::SEARCH; ViMotion::Down;
+        "h", +BindingMode::VI, ~BindingMode::SEARCH; ViMotion::Left;
+        "l", +BindingMode::VI, ~BindingMode::SEARCH; ViMotion::Right;
         Key::Named(ArrowUp), +BindingMode::VI; ViMotion::Up;
         Key::Named(ArrowDown), +BindingMode::VI; ViMotion::Down;
         Key::Named(ArrowLeft), +BindingMode::VI; ViMotion::Left;
@@ -615,201 +658,58 @@ pub fn default_key_bindings(
         Key::Named(ArrowDown), ModifiersState::SUPER, ~BindingMode::VI; Action::None;
         Key::Named(ArrowLeft), ModifiersState::SUPER, ~BindingMode::VI; Action::None;
         Key::Named(ArrowRight), ModifiersState::SUPER, ~BindingMode::VI; Action::None;
-        "0",                          +BindingMode::VI;
+        "0",                          +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::First;
-        "4",   ModifiersState::SHIFT, +BindingMode::VI;
+        "4",   ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::Last;
-        "6",   ModifiersState::SHIFT, +BindingMode::VI;
+        "6",   ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::FirstOccupied;
-        "h",      ModifiersState::SHIFT, +BindingMode::VI;
+        "h",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::High;
-        "m",      ModifiersState::SHIFT, +BindingMode::VI;
+        "m",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::Middle;
-        "l",      ModifiersState::SHIFT, +BindingMode::VI;
+        "l",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::Low;
-        "b",                             +BindingMode::VI;
+        "b",                             +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::SemanticLeft;
-        "w",                             +BindingMode::VI;
+        "w",                             +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::SemanticRight;
-        "e",                             +BindingMode::VI;
+        "e",                             +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::SemanticRightEnd;
-        "b",      ModifiersState::SHIFT, +BindingMode::VI;
+        "b",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::WordLeft;
-        "w",      ModifiersState::SHIFT, +BindingMode::VI;
+        "w",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::WordRight;
-        "e",      ModifiersState::SHIFT, +BindingMode::VI;
+        "e",      ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::WordRightEnd;
-        "5",   ModifiersState::SHIFT, +BindingMode::VI;
+        "5",   ModifiersState::SHIFT, +BindingMode::VI, ~BindingMode::SEARCH;
             ViMotion::Bracket;
     );
 
-    if !config_keyboard.use_kitty_keyboard_protocol {
-        bindings.extend(bindings!(
-            KeyBinding;
-            Key::Named(Home), ModifiersState::SHIFT, +BindingMode::ALT_SCREEN, ~BindingMode::VI; Action::Esc("\x1b[1;2H".into());
-            Key::Named(End), ModifiersState::SHIFT, +BindingMode::ALT_SCREEN, ~BindingMode::VI; Action::Esc("\x1b[1;2F".into());
-            Key::Named(End),  ~BindingMode::APP_CURSOR, ~BindingMode::VI; Action::Esc("\x1b[F".into());
-            Key::Named(PageUp), ModifiersState::SHIFT, +BindingMode::ALT_SCREEN, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[5;2~".into());
-            Key::Named(PageDown), ModifiersState::SHIFT, +BindingMode::ALT_SCREEN, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[6;2~".into());
-            Key::Named(Home),  ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[H".into());
-            Key::Named(ArrowUp), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[A".into());
-            Key::Named(ArrowDown), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[B".into());
-            Key::Named(ArrowRight), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[C".into());
-            Key::Named(ArrowLeft),  ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[D".into());
-            Key::Named(Backspace), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x7f".into());
-            Key::Named(Insert), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[2~".into());
-            Key::Named(Delete), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[3~".into());
-            Key::Named(PageUp), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[5~".into());
-            Key::Named(PageDown), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[6~".into());
-            Key::Named(F1), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1bOP".into());
-            Key::Named(F2), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1bOQ".into());
-            Key::Named(F3), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1bOR".into());
-            Key::Named(F4), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1bOS".into());
-            Key::Named(F5), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[15~".into());
-            Key::Named(F6), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[17~".into());
-            Key::Named(F7), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[18~".into());
-            Key::Named(F8), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[19~".into());
-            Key::Named(F9), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[20~".into());
-            Key::Named(F10), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[21~".into());
-            Key::Named(F11), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[23~".into());
-            Key::Named(F12), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[24~".into());
-            Key::Named(F13), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[25~".into());
-            Key::Named(F14), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[26~".into());
-            Key::Named(F15), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[28~".into());
-            Key::Named(F16), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[29~".into());
-            Key::Named(F17), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[31~".into());
-            Key::Named(F18), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[32~".into());
-            Key::Named(F19), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[33~".into());
-            Key::Named(F20), ~BindingMode::VI, ~BindingMode::SEARCH; Action::Esc("\x1b[34~".into());
-        ));
-
-        //   Code     Modifiers
-        // ---------+---------------------------
-        //    2     | Shift
-        //    3     | Alt
-        //    4     | Shift + Alt
-        //    5     | Control
-        //    6     | Shift + Control
-        //    7     | Alt + Control
-        //    8     | Shift + Alt + Control
-        // ---------+---------------------------
-        //
-        // from: https://invisible-island.net/xterm/ctlseqs/ctlseqs.html#h2-PC-Style-Function-Keys
-        let mut modifiers = vec![
-            ModifiersState::SHIFT,
-            ModifiersState::ALT,
-            ModifiersState::SHIFT | ModifiersState::ALT,
-            ModifiersState::CONTROL,
-            ModifiersState::SHIFT | ModifiersState::CONTROL,
-            ModifiersState::ALT | ModifiersState::CONTROL,
-            ModifiersState::SHIFT | ModifiersState::ALT | ModifiersState::CONTROL,
-        ];
-
-        for (index, mods) in modifiers.drain(..).enumerate() {
-            // If disable_ctlseqs_alt is enabled, should ignore ALT
-            // Useful for example if want same behaviour that Terminal.app have
-            // Since Terminal.app does not deal with ctlseqs with ALT keys
-            if index == 1 && config_keyboard.disable_ctlseqs_alt {
-                continue;
-            }
-
-            let modifiers_code = index + 2;
-            bindings.extend(bindings!(
-                KeyBinding;
-                Key::Named(Delete), mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[3;{}~", modifiers_code));
-                Key::Named(ArrowUp),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}A", modifiers_code));
-                Key::Named(ArrowDown),   mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}B", modifiers_code));
-                Key::Named(ArrowRight),  mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}C", modifiers_code));
-                Key::Named(ArrowLeft),   mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}D", modifiers_code));
-                Key::Named(F1),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}P", modifiers_code));
-                Key::Named(F2),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}Q", modifiers_code));
-                Key::Named(F3),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}R", modifiers_code));
-                Key::Named(F4),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[1;{}S", modifiers_code));
-                Key::Named(F5),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[15;{}~", modifiers_code));
-                Key::Named(F6),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[17;{}~", modifiers_code));
-                Key::Named(F7),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[18;{}~", modifiers_code));
-                Key::Named(F8),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[19;{}~", modifiers_code));
-                Key::Named(F9),     mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[20;{}~", modifiers_code));
-                Key::Named(F10),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[21;{}~", modifiers_code));
-                Key::Named(F11),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[23;{}~", modifiers_code));
-                Key::Named(F12),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[24;{}~", modifiers_code));
-                Key::Named(F13),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[25;{}~", modifiers_code));
-                Key::Named(F14),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[26;{}~", modifiers_code));
-                Key::Named(F15),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[28;{}~", modifiers_code));
-                Key::Named(F16),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[29;{}~", modifiers_code));
-                Key::Named(F17),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[31;{}~", modifiers_code));
-                Key::Named(F18),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[32;{}~", modifiers_code));
-                Key::Named(F19),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[33;{}~", modifiers_code));
-                Key::Named(F20),    mods, ~BindingMode::VI;
-                    Action::Esc(format!("\x1b[34;{}~", modifiers_code));
-            ));
-
-            // We're adding the following bindings with `Shift` manually above, so skipping them here.
-            if modifiers_code != 2 {
-                bindings.extend(bindings!(
-                    KeyBinding;
-                    Key::Named(Insert),   mods, ~BindingMode::VI;
-                        Action::Esc(format!("\x1b[2;{}~", modifiers_code));
-                    Key::Named(PageUp),   mods, ~BindingMode::VI;
-                        Action::Esc(format!("\x1b[5;{}~", modifiers_code));
-                    Key::Named(PageDown), mods, ~BindingMode::VI;
-                        Action::Esc(format!("\x1b[6;{}~", modifiers_code));
-                    Key::Named(End),      mods, ~BindingMode::VI;
-                        Action::Esc(format!("\x1b[1;{}F", modifiers_code));
-                    Key::Named(Home),     mods, ~BindingMode::VI;
-                        Action::Esc(format!("\x1b[1;{}H", modifiers_code));
-                ));
-            }
-        }
-    } else {
-        bindings.extend(bindings!(
-            KeyBinding;
-            Key::Named(ArrowUp), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[A".into());
-            Key::Named(ArrowDown), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[B".into());
-            Key::Named(ArrowRight), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[C".into());
-            Key::Named(ArrowLeft),  ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[D".into());
-            Key::Named(Insert),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[2~".into());
-            Key::Named(Delete),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[3~".into());
-            Key::Named(PageUp),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[5~".into());
-            Key::Named(PageDown),   ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[6~".into());
-            Key::Named(Backspace),  ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x7f".into());
-            Key::Named(Backspace), ModifiersState::ALT,     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b\x7f".into());
-            Key::Named(Backspace), ModifiersState::SHIFT,   ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x7f".into());
-            Key::Named(F1), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOP".into());
-            Key::Named(F2), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOQ".into());
-            Key::Named(F3), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOR".into());
-            Key::Named(F4), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOS".into());
-            Key::Named(Tab), ModifiersState::SHIFT, ~BindingMode::VI, ~BindingMode::SEARCH,   ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[Z".into());
-            Key::Named(Tab), ModifiersState::SHIFT | ModifiersState::ALT, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b\x1b[Z".into());
-        ));
-    }
+    bindings.extend(bindings!(
+        KeyBinding;
+        Key::Named(ArrowUp), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[A".into());
+        Key::Named(ArrowDown), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[B".into());
+        Key::Named(ArrowRight), ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[C".into());
+        Key::Named(ArrowLeft),  ~BindingMode::APP_CURSOR, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[D".into());
+        Key::Named(Insert),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[2~".into());
+        Key::Named(Delete),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[3~".into());
+        Key::Named(PageUp),     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[5~".into());
+        Key::Named(PageDown),   ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1b[6~".into());
+        Key::Named(Backspace),  ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x7f".into());
+        Key::Named(Backspace), ModifiersState::ALT,     ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b\x7f".into());
+        Key::Named(Backspace), ModifiersState::SHIFT,   ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x7f".into());
+        Key::Named(F1), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOP".into());
+        Key::Named(F2), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOQ".into());
+        Key::Named(F3), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOR".into());
+        Key::Named(F4), ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC, ~BindingMode::DISAMBIGUATE_KEYS; Action::Esc("\x1bOS".into());
+        Key::Named(Tab), ModifiersState::SHIFT, ~BindingMode::VI, ~BindingMode::SEARCH,   ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b[Z".into());
+        Key::Named(Tab), ModifiersState::SHIFT | ModifiersState::ALT, ~BindingMode::VI, ~BindingMode::SEARCH, ~BindingMode::ALL_KEYS_AS_ESC; Action::Esc("\x1b\x1b[Z".into());
+    ));
 
     bindings.extend(platform_key_bindings(
         use_navigation_key_bindings,
+        use_splits,
         config_keyboard,
     ));
 
@@ -1003,6 +903,7 @@ pub fn config_key_bindings(
 #[cfg(all(target_os = "macos", not(test)))]
 pub fn platform_key_bindings(
     use_navigation_key_bindings: bool,
+    use_splits: bool,
     config_keyboard: ConfigKeyboard,
 ) -> Vec<KeyBinding> {
     let mut key_bindings = bindings!(
@@ -1032,10 +933,6 @@ pub fn platform_key_bindings(
         // Search
         "f", ModifiersState::SUPER, ~BindingMode::SEARCH; Action::SearchForward;
         "b", ModifiersState::SUPER, ~BindingMode::SEARCH; Action::SearchBackward;
-        Key::Named(Escape), +BindingMode::SEARCH; SearchAction::SearchCancel;
-        Key::Named(Enter), +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusNext;
-        Key::Named(Enter), +BindingMode::SEARCH, +BindingMode::VI; SearchAction::SearchConfirm;
-        Key::Named(Enter), ModifiersState::SHIFT, +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusPrevious;
         "c", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchCancel;
         "u", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchClear;
         "w", ModifiersState::CONTROL,  +BindingMode::SEARCH; SearchAction::SearchDeleteWord;
@@ -1051,7 +948,7 @@ pub fn platform_key_bindings(
             "t", ModifiersState::SUPER; Action::TabCreateNew;
             Key::Named(Tab), ModifiersState::CONTROL; Action::SelectNextTab;
             Key::Named(Tab), ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectPrevTab;
-            "w", ModifiersState::SUPER; Action::TabCloseCurrent;
+            "w", ModifiersState::SUPER; Action::CloseCurrentSplitOrTab;
             "[", ModifiersState::SUPER | ModifiersState::SHIFT; Action::SelectPrevTab;
             "]", ModifiersState::SUPER | ModifiersState::SHIFT; Action::SelectNextTab;
             "1", ModifiersState::SUPER; Action::SelectTab(0);
@@ -1076,6 +973,16 @@ pub fn platform_key_bindings(
         ));
     }
 
+    if use_splits {
+        key_bindings.extend(bindings!(
+            KeyBinding;
+            "d", ModifiersState::SUPER, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitRight;
+            "d", ModifiersState::SUPER | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitDown;
+            "]", ModifiersState::SUPER, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectNextSplit;
+            "[", ModifiersState::SUPER, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectPrevSplit;
+        ));
+    }
+
     key_bindings
 }
 
@@ -1083,20 +990,21 @@ pub fn platform_key_bindings(
 #[cfg(not(any(target_os = "macos", target_os = "windows", test)))]
 pub fn platform_key_bindings(
     use_navigation_key_bindings: bool,
+    use_splits: bool,
     _: ConfigKeyboard,
 ) -> Vec<KeyBinding> {
     let mut key_bindings = bindings!(
         KeyBinding;
-        "v",        ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::VI; Action::Paste;
-        "c",        ModifiersState::CONTROL | ModifiersState::SHIFT; Action::Copy;
-        "c",        ModifiersState::CONTROL | ModifiersState::SHIFT,
+        "v", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::VI; Action::Paste;
+        "c", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::Copy;
+        "c", ModifiersState::CONTROL | ModifiersState::SHIFT,
             +BindingMode::VI; Action::ClearSelection;
         Key::Named(Insert),   ModifiersState::SHIFT, ~BindingMode::VI; Action::PasteSelection;
-        "0",     ModifiersState::CONTROL;  Action::ResetFontSize;
-        "=",   ModifiersState::CONTROL;  Action::IncreaseFontSize;
-        "+",     ModifiersState::CONTROL;  Action::IncreaseFontSize;
-        "+",      ModifiersState::CONTROL;  Action::IncreaseFontSize;
-        "-",          ModifiersState::CONTROL;  Action::DecreaseFontSize;
+        "0", ModifiersState::CONTROL;  Action::ResetFontSize;
+        "=", ModifiersState::CONTROL;  Action::IncreaseFontSize;
+        "+", ModifiersState::CONTROL;  Action::IncreaseFontSize;
+        "+", ModifiersState::CONTROL;  Action::IncreaseFontSize;
+        "-", ModifiersState::CONTROL;  Action::DecreaseFontSize;
         "-", ModifiersState::CONTROL;  Action::DecreaseFontSize;
         "n", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::WindowCreateNew;
         ",", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::ConfigEditor;
@@ -1104,10 +1012,6 @@ pub fn platform_key_bindings(
         // Search
         "f", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH; Action::SearchForward;
         "b", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH; Action::SearchBackward;
-        Key::Named(Escape), +BindingMode::SEARCH; SearchAction::SearchCancel;
-        Key::Named(Enter), +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusNext;
-        Key::Named(Enter), +BindingMode::SEARCH, +BindingMode::VI; SearchAction::SearchConfirm;
-        Key::Named(Enter), ModifiersState::SHIFT, +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusPrevious;
         "c", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchCancel;
         "u", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchClear;
         "w", ModifiersState::CONTROL,  +BindingMode::SEARCH; SearchAction::SearchDeleteWord;
@@ -1125,7 +1029,17 @@ pub fn platform_key_bindings(
             Key::Named(Tab), ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectPrevTab;
             "[", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectPrevTab;
             "]", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectNextTab;
-            "w", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::TabCloseCurrent;
+            "w", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::CloseCurrentSplitOrTab;
+        ));
+    }
+
+    if use_splits {
+        key_bindings.extend(bindings!(
+            KeyBinding;
+            "r", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitRight;
+            "d", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitDown;
+            "]", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectNextSplit;
+            "[", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectPrevSplit;
         ));
     }
 
@@ -1136,6 +1050,7 @@ pub fn platform_key_bindings(
 #[cfg(all(target_os = "windows", not(test)))]
 pub fn platform_key_bindings(
     use_navigation_key_bindings: bool,
+    use_splits: bool,
     _: ConfigKeyboard,
 ) -> Vec<KeyBinding> {
     let mut key_bindings = bindings!(
@@ -1162,10 +1077,6 @@ pub fn platform_key_bindings(
         // Search
         "f", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH; Action::SearchForward;
         "b", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH; Action::SearchBackward;
-        Key::Named(Escape), +BindingMode::SEARCH; SearchAction::SearchCancel;
-        Key::Named(Enter), +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusNext;
-        Key::Named(Enter), +BindingMode::SEARCH, +BindingMode::VI; SearchAction::SearchConfirm;
-        Key::Named(Enter), ModifiersState::SHIFT, +BindingMode::SEARCH, ~BindingMode::VI; SearchAction::SearchFocusPrevious;
         "c", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchCancel;
         "u", ModifiersState::CONTROL, +BindingMode::SEARCH; SearchAction::SearchClear;
         "w", ModifiersState::CONTROL,  +BindingMode::SEARCH; SearchAction::SearchDeleteWord;
@@ -1181,17 +1092,27 @@ pub fn platform_key_bindings(
             "t", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::TabCreateNew;
             Key::Named(Tab), ModifiersState::CONTROL; Action::SelectNextTab;
             Key::Named(Tab), ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectPrevTab;
-            "w", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::TabCloseCurrent;
+            "w", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::CloseCurrentSplitOrTab;
             "[", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectPrevTab;
             "]", ModifiersState::CONTROL | ModifiersState::SHIFT; Action::SelectNextTab;
         ));
     }
 
+    if use_splits {
+        key_bindings.extend(bindings!(
+            KeyBinding;
+            "r", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitRight;
+            "d", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SplitDown;
+            "]", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectNextSplit;
+            "[", ModifiersState::CONTROL | ModifiersState::SHIFT, ~BindingMode::SEARCH, ~BindingMode::VI; Action::SelectPrevSplit;
+        ));
+    }
+
     key_bindings
 }
 
 #[cfg(test)]
-pub fn platform_key_bindings(_: bool, _: ConfigKeyboard) -> Vec<KeyBinding> {
+pub fn platform_key_bindings(_: bool, _: bool, _: ConfigKeyboard) -> Vec<KeyBinding> {
     vec![]
 }
 
