@@ -115,9 +115,9 @@ impl Screen<'_> {
             1,
             config.window.macos_use_unified_titlebar,
         );
+
         let padding_y_bottom =
             padding_bottom_from_config(&config.navigation, config.padding_y[1], 1, false);
-
         let sugarloaf_layout =
             RootStyle::new(scale as f32, config.fonts.size, config.line_height);
 
@@ -450,11 +450,13 @@ impl Screen<'_> {
             self.clear_selection();
         }
         self.sugarloaf.resize(new_size.width, new_size.height);
-        self.context_manager
-            .current_grid_mut()
-            .resize(new_size.width as f32, new_size.height as f32);
+        let width = new_size.width as f32;
+        let height = new_size.height as f32;
+
+        for context_grid in self.context_manager.contexts_mut() {
+            context_grid.resize(width, height);
+        }
 
-        self.resize_all_contexts();
         self
     }
 
@@ -471,9 +473,12 @@ impl Screen<'_> {
         self.context_manager
             .current_grid_mut()
             .update_dimensions(&self.sugarloaf);
-        self.context_manager
-            .current_grid_mut()
-            .resize(new_size.width as f32, new_size.height as f32);
+        let width = new_size.width as f32;
+        let height = new_size.height as f32;
+
+        for context_grid in self.context_manager.contexts_mut() {
+            context_grid.resize(width, height);
+        }
 
         self
     }
@@ -997,6 +1002,18 @@ impl Screen<'_> {
                         self.context_manager.select_prev_split();
                         self.render();
                     }
+                    Act::SelectNextSplitOrTab => {
+                        self.cancel_search();
+                        self.clear_selection();
+                        self.context_manager.switch_to_next_split_or_tab();
+                        self.render();
+                    }
+                    Act::SelectPrevSplitOrTab => {
+                        self.cancel_search();
+                        self.clear_selection();
+                        self.context_manager.switch_to_prev_split_or_tab();
+                        self.render();
+                    }
                     Act::SelectTab(tab_index) => {
                         self.context_manager.select_tab(*tab_index);
                         self.cancel_search();
@@ -1065,12 +1082,15 @@ impl Screen<'_> {
     pub fn create_tab(&mut self) {
         let redirect = true;
 
+        // We resize the current tab ahead to prepare the
+        // dimensions to be copied to next tab.
+        let num_tabs = self.ctx().len();
+        self.resize_top_or_bottom_line(num_tabs + 1);
+
         let rich_text_id = self.sugarloaf.create_rich_text();
         self.context_manager.add_context(redirect, rich_text_id);
 
-        let num_tabs = self.ctx().len();
         self.cancel_search();
-        self.resize_top_or_bottom_line(num_tabs);
         self.render();
     }
 
@@ -1353,7 +1373,7 @@ impl Screen<'_> {
         let is_hyperlink_key_active = self.modifiers.state().super_key();
 
         #[cfg(not(target_os = "macos"))]
-        let is_hyperlink_key_active = self.modifiers.state().alt_key();
+        let is_hyperlink_key_active = self.modifiers.state().shift_key();
 
         if !is_hyperlink_key_active {
             return false;
@@ -1559,7 +1579,7 @@ impl Screen<'_> {
             .search_state
             .history
             .front()
-            .map_or(true, |regex| !regex.is_empty())
+            .is_none_or(|regex| !regex.is_empty())
         {
             self.search_state.history.push_front(String::new());
             self.search_state.history.truncate(MAX_SEARCH_HISTORY_SIZE);
@@ -2053,12 +2073,14 @@ impl Screen<'_> {
         self.sugarloaf.render();
     }
 
-    pub fn render_dialog(&mut self, content: &str) {
+    pub fn render_dialog(&mut self, content: &str, confirm: &str, close: &str) {
         self.sugarloaf.clear();
         crate::router::routes::dialog::screen(
             &mut self.sugarloaf,
             &self.context_manager.current().dimension,
             content,
+            confirm,
+            close,
         );
         self.sugarloaf.render();
     }
@@ -2087,13 +2109,12 @@ impl Screen<'_> {
             None
         };
 
-        self.renderer.prepare_term(
+        self.renderer.run(
             &mut self.sugarloaf,
             &mut self.context_manager,
             &mut search_hints,
             &self.search_state.focused_match,
         );
-        self.sugarloaf.render();
         // In this case the configuration of blinking cursor is enabled
         // and the terminal also have instructions of blinking enabled
         // TODO: enable blinking for selection after adding debounce (https://github.com/raphamorim/rio/issues/437)
